@@ -20,12 +20,13 @@
 // USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 var events = require('events');
-var stream = require('stream');
+var Stream = require('stream');
 var timers = require('timers');
 var util = require('util');
 var assert = require('assert');
+var cluster;
 
-function noop() {};
+function noop() {}
 
 // constructor for lazy loading
 function createPipe() {
@@ -44,7 +45,7 @@ function createTCP() {
 var FLAG_GOT_EOF = 1 << 0;
 var FLAG_SHUTDOWN = 1 << 1;
 var FLAG_DESTROY_SOON = 1 << 2;
-var FLAG_SHUTDOWNQUED = 1 << 3;
+var FLAG_SHUTDOWN_QUEUED = 1 << 3;
 
 
 var debug;
@@ -56,7 +57,7 @@ if (process.env.NODE_DEBUG && /net/.test(process.env.NODE_DEBUG)) {
 
 
 function isPipeName(s) {
-  return typeof s === 'string' && toPort(s) === false;
+  return typeof s === 'string' && toNumber(s) === false;
 }
 
 
@@ -65,17 +66,48 @@ exports.createServer = function() {
 };
 
 
-exports.connect = exports.createConnection = function(port /* [host], [cb] */) {
-  var s;
+// Target API:
+//
+// var s = net.connect({port: 80, host: 'google.com'}, function() {
+//   ...
+// });
+//
+// There are various forms:
+//
+// connect(options, [cb])
+// connect(port, [host], [cb])
+// connect(path, [cb]);
+//
+exports.connect = exports.createConnection = function() {
+  var args = normalizeConnectArgs(arguments);
+  var s = new Socket(args[0]);
+  return Socket.prototype.connect.apply(s, args);
+};
 
-  if (isPipeName(port)) {
-    s = new Socket({ handle: createPipe() });
+// Returns an array [options] or [options, cb]
+// It is the same as the argument of Socket.prototype.connect().
+function normalizeConnectArgs(args) {
+  var options = {};
+
+  if (typeof args[0] === 'object') {
+    // connect(options, [cb])
+    options = args[0];
+  } else if (isPipeName(args[0])) {
+    // connect(path, [cb]);
+    options.path = args[0];
   } else {
-    s = new Socket();
+    // connect(port, [host], [cb])
+    options.port = args[0];
+    if (typeof args[1] === 'string') {
+      options.host = args[1];
+    }
   }
 
-  return s.connect(port, arguments[1], arguments[2]);
-};
+  var cb = args[args.length - 1];
+  return (typeof cb === 'function') ? [options, cb] : [options];
+}
+exports._normalizeConnectArgs = normalizeConnectArgs;
+
 
 /* called when creating new Socket, or when re-using a closed Socket */
 function initSocketHandle(self) {
@@ -86,11 +118,11 @@ function initSocketHandle(self) {
   self.destroyed = false;
   self.errorEmitted = false;
   self.bytesRead = 0;
-  self.bytesWritten = 0;
+  self._bytesDispatched = 0;
 
   // Handle creation may be deferred to bind() or connect() time.
   if (self._handle) {
-    self._handle.socket = self;
+    self._handle.owner = self;
     self._handle.onread = onread;
   }
 }
@@ -98,30 +130,29 @@ function initSocketHandle(self) {
 function Socket(options) {
   if (!(this instanceof Socket)) return new Socket(options);
 
-  stream.Stream.call(this);
+  Stream.call(this);
 
-  if (typeof options == 'number') {
-    // Legacy interface.
-    // Must support legacy interface. Old versions of npm depend on it.
-    // https://github.com/isaacs/npm/blob/c7824f412f0cb59d6f55cf0bc220253c39e6029f/lib/utils/output.js#L110
-    var fd = options;
-
-    // Uncomment the following lines after libuv backend is stable and API
-    // compatibile with legaacy.
-    //   console.error('Deprecated interface net.Socket(fd).');
-    //   console.trace();
-    this._handle = createPipe();
-    this._handle.open(fd);
-    this.readable = this.writable = true;
-    initSocketHandle(this);
-  } else {
-    // private
-    this._handle = options && options.handle;
-    initSocketHandle(this);
-    this.allowHalfOpen = options && options.allowHalfOpen;
+  switch (typeof options) {
+    case 'number':
+      options = { fd: options }; // Legacy interface.
+      break;
+    case 'undefined':
+      options = {};
+      break;
   }
+
+  if (typeof options.fd === 'undefined') {
+    this._handle = options && options.handle; // private
+  } else {
+    this._handle = createPipe();
+    this._handle.open(options.fd);
+    this.readable = this.writable = true;
+  }
+
+  initSocketHandle(this);
+  this.allowHalfOpen = options && options.allowHalfOpen;
 }
-util.inherits(Socket, stream.Stream);
+util.inherits(Socket, Stream);
 
 
 exports.Socket = Socket;
@@ -131,12 +162,12 @@ exports.Stream = Socket; // Legacy naming.
 Socket.prototype.listen = function() {
   var self = this;
   self.on('connection', arguments[0]);
-  listen(self, null, null);
+  listen(self, null, null, null);
 };
 
 
 Socket.prototype.setTimeout = function(msecs, callback) {
-  if (msecs > 0) {
+  if (msecs > 0 && !isNaN(msecs) && isFinite(msecs)) {
     timers.enroll(this, msecs);
     timers.active(this);
     if (callback) {
@@ -144,19 +175,23 @@ Socket.prototype.setTimeout = function(msecs, callback) {
     }
   } else if (msecs === 0) {
     timers.unenroll(this);
+    if (callback) {
+      this.removeListener('timeout', callback);
+    }
   }
 };
 
 
 Socket.prototype._onTimeout = function() {
-  debug("_onTimeout");
+  debug('_onTimeout');
   this.emit('timeout');
 };
 
 
-Socket.prototype.setNoDelay = function() {
+Socket.prototype.setNoDelay = function(enable) {
+  // backwards compatibility: assume true when `enable` is omitted
   if (this._handle && this._handle.setNoDelay)
-    this._handle.setNoDelay();
+    this._handle.setNoDelay(typeof enable === 'undefined' ? true : !!enable);
 };
 
 
@@ -201,35 +236,27 @@ Object.defineProperty(Socket.prototype, 'bufferSize', {
 
 
 Socket.prototype.pause = function() {
-  if (this._handle) {
+  this._paused = true;
+  if (this._handle && !this._connecting) {
     this._handle.readStop();
-
-    // this adds an undesireable boundary crossing for pipe streams.
-    // the .unref() method is omitted on TCP streams, because it is
-    // unnecessary.
-    if (this._handle.unref) this._handle.unref();
   }
 };
 
 
 Socket.prototype.resume = function() {
-  if (this._handle) {
+  this._paused = false;
+  if (this._handle && !this._connecting) {
     this._handle.readStart();
-
-    // this adds an undesireable boundary crossing for pipe streams.
-    // the .ref() method is omitted on TCP streams, because it is
-    // unnecessary.
-    if (this._handle.ref) this._handle.ref();
   }
 };
 
 
 Socket.prototype.end = function(data, encoding) {
-  if (this._connecting && ((this._flags & FLAG_SHUTDOWNQUED) == 0)) {
+  if (this._connecting && ((this._flags & FLAG_SHUTDOWN_QUEUED) == 0)) {
     // still connecting, add data to buffer
     if (data) this.write(data, encoding);
     this.writable = false;
-    this._flags |= FLAG_SHUTDOWNQUED;
+    this._flags |= FLAG_SHUTDOWN_QUEUED;
   }
 
   if (!this.writable) return;
@@ -257,7 +284,7 @@ Socket.prototype.end = function(data, encoding) {
 
 
 function afterShutdown(status, handle, req) {
-  var self = handle.socket;
+  var self = handle.owner;
 
   assert.ok(self._flags & FLAG_SHUTDOWN);
   assert.ok(!self.writable);
@@ -317,11 +344,6 @@ Socket.prototype._destroy = function(exception, cb) {
 
   timers.unenroll(this);
 
-  if (this.server) {
-    this.server.connections--;
-    this.server._emitCloseIfDrained();
-  }
-
   debug('close');
   if (this._handle) {
     this._handle.close();
@@ -336,18 +358,25 @@ Socket.prototype._destroy = function(exception, cb) {
   });
 
   this.destroyed = true;
+
+  if (this.server) {
+    this.server._connections--;
+    if (this.server._emitCloseIfDrained) {
+      this.server._emitCloseIfDrained();
+    }
+  }
 };
 
 
 Socket.prototype.destroy = function(exception) {
   this._destroy(exception);
-}
+};
 
 
 function onread(buffer, offset, length) {
   var handle = this;
-  var self = handle.socket;
-  assert.equal(handle, self._handle);
+  var self = handle.owner;
+  assert(handle === self._handle, 'handle != self._handle');
 
   timers.active(self);
 
@@ -410,6 +439,10 @@ Socket.prototype._getpeername = function() {
   }
   if (!this._peername) {
     this._peername = this._handle.getpeername();
+    // getpeername() returns null on error
+    if (this._peername === null) {
+      return {};
+    }
   }
   return this._peername;
 };
@@ -439,18 +472,29 @@ Socket.prototype.write = function(data, arg1, arg2) {
     } else if (typeof arg1 === 'function') {
       cb = arg1;
     } else {
-      throw new Error("bad arg");
+      throw new Error('bad arg');
     }
   }
 
-  // Change strings to buffers. SLOW
   if (typeof data === 'string') {
-    data = new Buffer(data, encoding);
-  } else if (!Buffer.isBuffer(data)) {
-    throw new TypeError("First argument must be a buffer or a string.");
-  }
+    encoding = (encoding || 'utf8').toLowerCase();
+    switch (encoding) {
+      case 'utf8':
+      case 'utf-8':
+      case 'ascii':
+      case 'ucs2':
+      case 'ucs-2':
+      case 'utf16le':
+      case 'utf-16le':
+        // This encoding can be handled in the binding layer.
+        break;
 
-  this.bytesWritten += data.length;
+      default:
+        data = new Buffer(data, encoding);
+    }
+  } else if (!Buffer.isBuffer(data)) {
+    throw new TypeError('First argument must be a buffer or a string.');
+  }
 
   // If we are still connecting, then buffer this for later.
   if (this._connecting) {
@@ -475,24 +519,70 @@ Socket.prototype._write = function(data, encoding, cb) {
     return false;
   }
 
-  // `encoding` is unused right now, `data` is always a buffer.
-  var writeReq = this._handle.write(data);
+  var writeReq;
 
-  if (!writeReq) {
+  if (Buffer.isBuffer(data)) {
+    writeReq = this._handle.writeBuffer(data);
+
+  } else {
+    switch (encoding) {
+      case 'utf8':
+      case 'utf-8':
+        writeReq = this._handle.writeUtf8String(data);
+        break;
+
+      case 'ascii':
+        writeReq = this._handle.writeAsciiString(data);
+        break;
+
+      case 'ucs2':
+      case 'ucs-2':
+      case 'utf16le':
+      case 'utf-16le':
+        writeReq = this._handle.writeUcs2String(data);
+        break;
+
+      default:
+        assert(0);
+    }
+  }
+
+  if (!writeReq || typeof writeReq !== 'object') {
     this._destroy(errnoException(errno, 'write'), cb);
     return false;
   }
 
   writeReq.oncomplete = afterWrite;
   writeReq.cb = cb;
+
   this._pendingWriteReqs++;
+  this._bytesDispatched += writeReq.bytes;
 
   return this._handle.writeQueueSize == 0;
 };
 
 
-function afterWrite(status, handle, req, buffer) {
-  var self = handle.socket;
+Socket.prototype.__defineGetter__('bytesWritten', function() {
+  var bytes = this._bytesDispatched,
+      connectQueue = this._connectQueue;
+
+  if (connectQueue) {
+    connectQueue.forEach(function(el) {
+      var data = el[0];
+      if (Buffer.isBuffer(data)) {
+        bytes += data.length;
+      } else {
+        bytes += Buffer.byteLength(data, el[1]);
+      }
+    }, this);
+  }
+
+  return bytes;
+});
+
+
+function afterWrite(status, handle, req) {
+  var self = handle.owner;
 
   // callback may come after call to destroy.
   if (self.destroyed) {
@@ -509,8 +599,6 @@ function afterWrite(status, handle, req, buffer) {
   self._pendingWriteReqs--;
 
   if (self._pendingWriteReqs == 0) {
-    // TODO remove all uses of ondrain - this is not a good hack.
-    if (self.ondrain) self.ondrain();
     self.emit('drain');
   }
 
@@ -522,17 +610,26 @@ function afterWrite(status, handle, req, buffer) {
 }
 
 
-function connect(self, address, port, addressType) {
-  if (port) {
-    self.remotePort = port;
-  }
-  self.remoteAddress = address;
-
+function connect(self, address, port, addressType, localAddress) {
   // TODO return promise from Socket.prototype.connect which
   // wraps _connectReq.
 
   assert.ok(self._connecting);
 
+  if (localAddress) {
+    var r;
+    if (addressType == 6) {
+      r = self._handle.bind6(localAddress);
+    } else {
+      r = self._handle.bind(localAddress);
+    }
+
+    if (r) {
+      self._destroy(errnoException(errno, 'bind'));
+      return;
+    }
+  }
+
   var connectReq;
   if (addressType == 6) {
     connectReq = self._handle.connect6(address, port);
@@ -550,24 +647,25 @@ function connect(self, address, port, addressType) {
 }
 
 
-Socket.prototype.connect = function(port /* [host], [cb] */) {
-  var self = this;
+Socket.prototype.connect = function(options, cb) {
+  if (typeof options !== 'object') {
+    // Old API:
+    // connect(port, [host], [cb])
+    // connect(path, [cb]);
+    var args = normalizeConnectArgs(arguments);
+    return Socket.prototype.connect.apply(this, args);
+  }
 
-  var pipe = isPipeName(port);
+  var self = this;
+  var pipe = !!options.path;
 
   if (this.destroyed || !this._handle) {
     this._handle = pipe ? createPipe() : createTCP();
     initSocketHandle(this);
   }
 
-  var host;
-  if (typeof arguments[1] === 'function') {
-    self.on('connect', arguments[1]);
-  } else {
-    host = arguments[1];
-    if (typeof arguments[2] === 'function') {
-      self.on('connect', arguments[2]);
-    }
+  if (typeof cb === 'function') {
+    self.once('connect', cb);
   }
 
   timers.active(this);
@@ -576,9 +674,14 @@ Socket.prototype.connect = function(port /* [host], [cb] */) {
   self.writable = true;
 
   if (pipe) {
-    connect(self, /*pipe_name=*/port);
+    connect(self, options.path);
 
-  } else if (typeof host == 'string') {
+  } else if (!options.host) {
+    debug('connect: missing host');
+    connect(self, '127.0.0.1', options.port, 4);
+
+  } else {
+    var host = options.host;
     debug('connect: find host ' + host);
     require('dns').lookup(host, function(err, ip, addressType) {
       // It's possible we were destroyed while looking this up.
@@ -604,27 +707,23 @@ Socket.prototype.connect = function(port /* [host], [cb] */) {
         // expects remoteAddress to have a meaningful value
         ip = ip || (addressType === 4 ? '127.0.0.1' : '0:0:0:0:0:0:0:1');
 
-        connect(self, ip, port, addressType);
+        connect(self, ip, options.port, addressType, options.localAddress);
       }
     });
-
-  } else {
-    debug('connect: missing host');
-    connect(self, '127.0.0.1', port, 4);
   }
   return self;
 };
 
 
 function afterConnect(status, handle, req, readable, writable) {
-  var self = handle.socket;
+  var self = handle.owner;
 
   // callback may come after call to destroy
   if (self.destroyed) {
     return;
   }
 
-  assert.equal(handle, self._handle);
+  assert(handle === self._handle, 'handle != self._handle');
 
   debug('afterConnect');
 
@@ -636,7 +735,7 @@ function afterConnect(status, handle, req, readable, writable) {
     self.writable = writable;
     timers.active(self);
 
-    if (self.readable) {
+    if (self.readable && !self._paused) {
       handle.readStart();
     }
 
@@ -651,9 +750,9 @@ function afterConnect(status, handle, req, readable, writable) {
 
     self.emit('connect');
 
-    if (self._flags & FLAG_SHUTDOWNQUED) {
+    if (self._flags & FLAG_SHUTDOWN_QUEUED) {
       // end called before connected - call end now with no data
-      self._flags &= ~FLAG_SHUTDOWNQUED;
+      self._flags &= ~FLAG_SHUTDOWN_QUEUED;
       self.end();
     }
   } else {
@@ -695,7 +794,23 @@ function Server(/* [ options, ] listener */) {
     }
   }
 
-  this.connections = 0;
+  this._connections = 0;
+
+  // when server is using slaves .connections is not reliable
+  // so null will be return if thats the case
+  Object.defineProperty(this, 'connections', {
+    get: function() {
+      if (self._usingSlaves) {
+        return null;
+      }
+      return self._connections;
+    },
+    set: function(val) {
+      return (self._connections = val);
+    },
+    configurable: true, enumerable: true
+  });
+
   this.allowHalfOpen = options.allowHalfOpen || false;
 
   this._handle = null;
@@ -704,16 +819,38 @@ util.inherits(Server, events.EventEmitter);
 exports.Server = Server;
 
 
-function toPort(x) { return (x = Number(x)) >= 0 ? x : false; }
+function toNumber(x) { return (x = Number(x)) >= 0 ? x : false; }
 
 
 var createServerHandle = exports._createServerHandle =
-    function(address, port, addressType) {
+    function(address, port, addressType, fd) {
   var r = 0;
   // assign handle in listen, and clean up if bind or listen fails
   var handle;
 
-  if (port == -1 && addressType == -1) {
+  if (typeof fd === 'number' && fd >= 0) {
+    var tty_wrap = process.binding('tty_wrap');
+    var type = tty_wrap.guessHandleType(fd);
+    switch (type) {
+      case 'PIPE':
+        debug('listen pipe fd=' + fd);
+        // create a PipeWrap
+        handle = createPipe();
+        handle.open(fd);
+        handle.readable = true;
+        handle.writable = true;
+        break;
+
+      default:
+        // Not a fd we can listen on.  This will trigger an error.
+        debug('listen invalid fd=' + fd + ' type=' + type);
+        global.errno = 'EINVAL'; // hack, callers expect that errno is set
+        handle = null;
+        break;
+    }
+    return handle;
+
+  } else if (port == -1 && addressType == -1) {
     handle = createPipe();
     if (process.platform === 'win32') {
       var instances = parseInt(process.env.NODE_PENDING_PIPE_INSTANCES);
@@ -743,50 +880,60 @@ var createServerHandle = exports._createServerHandle =
 };
 
 
-Server.prototype._listen2 = function(address, port, addressType) {
+Server.prototype._listen2 = function(address, port, addressType, backlog, fd) {
   var self = this;
   var r = 0;
 
   // If there is not yet a handle, we need to create one and bind.
   // In the case of a server sent via IPC, we don't need to do this.
   if (!self._handle) {
-    self._handle = createServerHandle(address, port, addressType);
+    self._handle = createServerHandle(address, port, addressType, fd);
     if (!self._handle) {
+      var error = errnoException(errno, 'listen');
       process.nextTick(function() {
-        self.emit('error', errnoException(errno, 'listen'));
+        self.emit('error', error);
       });
       return;
     }
   }
 
   self._handle.onconnection = onconnection;
-  self._handle.socket = self;
+  self._handle.owner = self;
 
-  r = self._handle.listen(self._backlog || 128);
+  // Use a backlog of 512 entries. We pass 511 to the listen() call because
+  // the kernel does: backlogsize = roundup_pow_of_two(backlogsize + 1);
+  // which will thus give us a backlog of 512 entries.
+  r = self._handle.listen(backlog || 511);
 
   if (r) {
+    var ex = errnoException(errno, 'listen');
     self._handle.close();
     self._handle = null;
     process.nextTick(function() {
-      self.emit('error', errnoException(errno, 'listen'));
+      self.emit('error', ex);
     });
     return;
   }
 
+  // generate connection key, this should be unique to the connection
+  this._connectionKey = addressType + ':' + address + ':' + port;
+
   process.nextTick(function() {
     self.emit('listening');
   });
-}
+};
 
 
-function listen(self, address, port, addressType) {
-  if (process.env.NODE_WORKER_ID) {
-    require('cluster')._getServer(address, port, addressType, function(handle) {
+function listen(self, address, port, addressType, backlog, fd) {
+  if (!cluster) cluster = require('cluster');
+
+  if (cluster.isWorker) {
+    cluster._getServer(self, address, port, addressType, fd, function(handle) {
       self._handle = handle;
-      self._listen2(address, port, addressType);
+      self._listen2(address, port, addressType, backlog, fd);
     });
   } else {
-    self._listen2(address, port, addressType);
+    self._listen2(address, port, addressType, backlog, fd);
   }
 }
 
@@ -799,36 +946,51 @@ Server.prototype.listen = function() {
     self.once('listening', lastArg);
   }
 
-  var port = toPort(arguments[0]);
+  var port = toNumber(arguments[0]);
+
+  // The third optional argument is the backlog size.
+  // When the ip is omitted it can be the second argument.
+  var backlog = toNumber(arguments[1]) || toNumber(arguments[2]);
 
   var TCP = process.binding('tcp_wrap').TCP;
 
   if (arguments.length == 0 || typeof arguments[0] == 'function') {
-    // Don't bind(). OS will assign a port with INADDR_ANY.
-    // The port can be found with server.address()
-    listen(self, null, null);
-
-  } else if (arguments[0] instanceof TCP) {
-    self._handle = arguments[0];
-    listen(self, null, -1, -1);
+    // Bind to a random port.
+    listen(self, '0.0.0.0', 0, null, backlog);
 
+  } else if (arguments[0] && typeof arguments[0] === 'object') {
+    var h = arguments[0];
+    if (h._handle) {
+      h = h._handle;
+    } else if (h.handle) {
+      h = h.handle;
+    }
+    if (h instanceof TCP) {
+      self._handle = h;
+      listen(self, null, -1, -1, backlog);
+    } else if (typeof h.fd === 'number' && h.fd >= 0) {
+      listen(self, null, null, null, backlog, h.fd);
+    } else {
+      throw new Error('Invalid listen argument: ' + h);
+    }
   } else if (isPipeName(arguments[0])) {
     // UNIX socket or Windows pipe.
     var pipeName = self._pipeName = arguments[0];
-    listen(self, pipeName, -1, -1);
+    listen(self, pipeName, -1, -1, backlog);
 
   } else if (typeof arguments[1] == 'undefined' ||
-             typeof arguments[1] == 'function') {
+             typeof arguments[1] == 'function' ||
+             typeof arguments[1] == 'number') {
     // The first argument is the port, no IP given.
-    listen(self, '0.0.0.0', port, 4);
+    listen(self, '0.0.0.0', port, 4, backlog);
 
   } else {
-    // The first argument is the port, the second an IP
+    // The first argument is the port, the second an IP.
     require('dns').lookup(arguments[1], function(err, ip, addressType) {
       if (err) {
         self.emit('error', err);
       } else {
-        listen(self, ip || '0.0.0.0', port, ip ? addressType : 4);
+        listen(self, ip || '0.0.0.0', port, ip ? addressType : 4, backlog);
       }
     });
   }
@@ -847,7 +1009,7 @@ Server.prototype.address = function() {
 
 function onconnection(clientHandle) {
   var handle = this;
-  var self = handle.socket;
+  var self = handle.owner;
 
   debug('onconnection');
 
@@ -856,7 +1018,7 @@ function onconnection(clientHandle) {
     return;
   }
 
-  if (self.maxConnections && self.connections >= self.maxConnections) {
+  if (self.maxConnections && self._connections >= self.maxConnections) {
     clientHandle.close();
     return;
   }
@@ -867,9 +1029,9 @@ function onconnection(clientHandle) {
   });
   socket.readable = socket.writable = true;
 
-  socket.resume();
+  clientHandle.readStart();
 
-  self.connections++;
+  self._connections++;
   socket.server = self;
 
   DTRACE_NET_SERVER_CONNECTION(socket);
@@ -878,28 +1040,52 @@ function onconnection(clientHandle) {
 }
 
 
-Server.prototype.close = function() {
+Server.prototype.close = function(cb) {
   if (!this._handle) {
     // Throw error. Follows net_legacy behaviour.
     throw new Error('Not running');
   }
 
+  if (cb) {
+    this.once('close', cb);
+  }
   this._handle.close();
   this._handle = null;
   this._emitCloseIfDrained();
 
+  // fetch new socket lists
+  if (this._usingSlaves) {
+    this._slaves.forEach(function(socketList) {
+      if (socketList.list.length === 0) return;
+      socketList.update();
+    });
+  }
+
   return this;
 };
 
 Server.prototype._emitCloseIfDrained = function() {
-  if (!this._handle && !this.connections) {
-    this.emit('close');
-  }
+  var self = this;
+
+  if (self._handle || self._connections) return;
+
+  process.nextTick(function() {
+    self.emit('close');
+  });
 };
 
 
-Server.prototype.listenFD = function(fd, type) {
-  throw new Error('This API is no longer supported. See child_process.fork');
+Server.prototype.listenFD = util.deprecate(function(fd, type) {
+  return this.listen({ fd: fd });
+}, 'listenFD is deprecated. Use listen({fd: <number>}).');
+
+// when sending a socket using fork IPC this function is executed
+Server.prototype._setupSlave = function(socketList) {
+  if (!this._usingSlaves) {
+    this._usingSlaves = true;
+    this._slaves = [];
+  }
+  this._slaves.push(socketList);
 };
 
 
@@ -909,7 +1095,7 @@ Server.prototype.listenFD = function(fd, type) {
 //  and it does not detect more than one double : in a string.
 exports.isIP = function(input) {
   if (!input) {
-    return 4;
+    return 0;
   } else if (/^(\d?\d?\d)\.(\d?\d?\d)\.(\d?\d?\d)\.(\d?\d?\d)$/.test(input)) {
     var parts = input.split('.');
     for (var i = 0; i < parts.length; i++) {
@@ -936,3 +1122,26 @@ exports.isIPv4 = function(input) {
 exports.isIPv6 = function(input) {
   return exports.isIP(input) === 6;
 };
+
+
+if (process.platform === 'win32') {
+  var simultaneousAccepts;
+
+  exports._setSimultaneousAccepts = function(handle) {
+    if (typeof handle === 'undefined') {
+      return;
+    }
+
+    if (typeof simultaneousAccepts === 'undefined') {
+      simultaneousAccepts = (process.env.NODE_MANY_ACCEPTS &&
+                             process.env.NODE_MANY_ACCEPTS !== '0');
+    }
+
+    if (handle._simultaneousAccepts !== simultaneousAccepts) {
+      handle.setSimultaneousAccepts(simultaneousAccepts);
+      handle._simultaneousAccepts = simultaneousAccepts;
+    }
+  };
+} else {
+  exports._setSimultaneousAccepts = function(handle) {};
+}
