@@ -27,12 +27,12 @@
  *   repl.start("prompt> ");
  *
  *   // listen for unix socket connections and start repl on them
- *   net.createServer(function (socket) {
+ *   net.createServer(function(socket) {
  *     repl.start("node via Unix socket> ", socket);
  *   }).listen("/tmp/node-repl-sock");
  *
  *   // listen for TCP socket connections and start repl on them
- *   net.createServer(function (socket) {
+ *   net.createServer(function(socket) {
  *     repl.start("node via TCP socket> ", socket);
  *   }).listen(5001);
  *
@@ -41,10 +41,13 @@
  */
 
 var util = require('util');
+var inherits = require('util').inherits;
+var Stream = require('stream');
 var vm = require('vm');
 var path = require('path');
 var fs = require('fs');
 var rl = require('readline');
+var EventEmitter = require('events').EventEmitter;
 
 // If obj.hasOwnProperty has been overridden, then calling
 // obj.hasOwnProperty(prop) will break.
@@ -54,29 +57,55 @@ function hasOwnProperty(obj, prop) {
 }
 
 
-var context;
-
-exports.disableColors = process.env.NODE_DISABLE_COLORS ? true : false;
-
 // hack for require.resolve("./relative") to work properly.
-module.filename = process.cwd() + '/repl';
+module.filename = path.resolve('repl');
 
 // hack for repl require to work properly with node_modules folders
 module.paths = require('module')._nodeModulePaths(module.filename);
 
-// Can overridden with custom print functions, such as `probe` or `eyes.js`
+// Can overridden with custom print functions, such as `probe` or `eyes.js`.
+// This is the default "writer" value if none is passed in the REPL options.
 exports.writer = util.inspect;
 
+exports._builtinLibs = ['assert', 'buffer', 'child_process', 'cluster',
+  'crypto', 'dgram', 'dns', 'events', 'fs', 'http', 'https', 'net',
+  'os', 'path', 'punycode', 'querystring', 'readline', 'repl',
+  'string_decoder', 'tls', 'tty', 'url', 'util', 'vm', 'zlib'];
+
+
+function REPLServer(prompt, stream, eval_, useGlobal, ignoreUndefined) {
+  if (!(this instanceof REPLServer)) {
+    return new REPLServer(prompt, stream, eval_, useGlobal, ignoreUndefined);
+  }
+
+  EventEmitter.call(this);
+
+  var options, input, output;
+  if (typeof prompt == 'object') {
+    // an options object was given
+    options = prompt;
+    stream = options.stream || options.socket;
+    input = options.input;
+    output = options.output;
+    eval_ = options.eval;
+    useGlobal = options.useGlobal;
+    ignoreUndefined = options.ignoreUndefined;
+    prompt = options.prompt;
+  } else if (typeof prompt != 'string') {
+    throw new Error('An options Object, or a prompt String are required');
+  } else {
+    options = {};
+  }
 
-function REPLServer(prompt, stream, eval, useGlobal, ignoreUndefined) {
   var self = this;
 
-  self.useGlobal = useGlobal;
+  self.useGlobal = !!useGlobal;
+  self.ignoreUndefined = !!ignoreUndefined;
 
-  self.eval = eval || function(code, context, file, cb) {
+  self.eval = eval_ || function(code, context, file, cb) {
     var err, result;
     try {
-      if (useGlobal) {
+      if (self.useGlobal) {
         result = vm.runInThisContext(code, file);
       } else {
         result = vm.runInContext(code, context, file);
@@ -90,65 +119,87 @@ function REPLServer(prompt, stream, eval, useGlobal, ignoreUndefined) {
   self.resetContext();
   self.bufferedCommand = '';
 
-  if (stream) {
-    // We're given a duplex socket
-    if (stream.stdin || stream.stdout) {
-      self.outputStream = stream.stdout;
-      self.inputStream = stream.stdin;
+  if (!input && !output) {
+    // legacy API, passing a 'stream'/'socket' option
+    if (!stream) {
+      // use stdin and stdout as the default streams if none were given
+      stream = process;
+    }
+    if (stream.stdin && stream.stdout) {
+      // We're given custom object with 2 streams, or the `process` object
+      input = stream.stdin;
+      output = stream.stdout;
     } else {
-      self.outputStream = stream;
-      self.inputStream = stream;
+      // We're given a duplex readable/writable Stream, like a `net.Socket`
+      input = stream;
+      output = stream;
     }
-  } else {
-    self.outputStream = process.stdout;
-    self.inputStream = process.stdin;
-    process.stdin.resume();
   }
 
+
+  self.inputStream = input;
+  self.outputStream = output;
+
   self.prompt = (prompt != undefined ? prompt : '> ');
 
   function complete(text, callback) {
     self.complete(text, callback);
   }
 
-  var rli = rl.createInterface(self.inputStream, self.outputStream, complete);
+  var rli = rl.createInterface({
+    input: self.inputStream,
+    output: self.outputStream,
+    completer: complete,
+    terminal: options.terminal
+  });
   self.rli = rli;
 
   this.commands = {};
   defineDefaultCommands(this);
 
-  if (rli.enabled && !exports.disableColors &&
-      exports.writer === util.inspect) {
+  // figure out which "writer" function to use
+  self.writer = options.writer || exports.writer;
+
+  if (typeof options.useColors === 'undefined') {
+    options.useColors = rli.terminal;
+  }
+  self.useColors = !!options.useColors;
+
+  if (self.useColors && self.writer === util.inspect) {
     // Turn on ANSI coloring.
-    exports.writer = function(obj, showHidden, depth) {
+    self.writer = function(obj, showHidden, depth) {
       return util.inspect(obj, showHidden, depth, true);
     };
   }
 
   rli.setPrompt(self.prompt);
 
+  rli.on('close', function() {
+    self.emit('exit');
+  });
+
   var sawSIGINT = false;
   rli.on('SIGINT', function() {
-    if (sawSIGINT) {
-      rli.close();
-      process.exit();
-    }
+    var empty = rli.line.length === 0;
+    rli.clearLine();
 
-    rli.line = '';
-
-    if (!(self.bufferedCommand && self.bufferedCommand.length > 0) &&
-        rli.line.length === 0) {
-      rli.output.write('\n(^C again to quit)\n');
+    if (!(self.bufferedCommand && self.bufferedCommand.length > 0) && empty) {
+      if (sawSIGINT) {
+        rli.close();
+        sawSIGINT = false;
+        return;
+      }
+      rli.output.write('(^C again to quit)\n');
       sawSIGINT = true;
     } else {
-      rli.output.write('\n');
+      sawSIGINT = false;
     }
 
     self.bufferedCommand = '';
     self.displayPrompt();
   });
 
-  rli.addListener('line', function(cmd) {
+  rli.on('line', function(cmd) {
     sawSIGINT = false;
     var skipCatchall = false;
     cmd = trimWhitespace(cmd);
@@ -167,6 +218,21 @@ function REPLServer(prompt, stream, eval, useGlobal, ignoreUndefined) {
       }
     }
 
+    // Check if a builtin module name was used and then include it
+    // if there's no conflict.
+    if (exports._builtinLibs.indexOf(cmd) !== -1) {
+      var lib = require(cmd);
+      if (cmd in self.context && lib !== self.context[cmd]) {
+        self.outputStream.write('A different "' + cmd +
+                                '" already exists globally\n');
+      } else {
+        self.context._ = self.context[cmd] = lib;
+        self.outputStream.write(self.writer(lib) + '\n');
+      }
+      self.displayPrompt();
+      return;
+    }
+
     if (!skipCatchall) {
       var evalCmd = self.bufferedCommand + cmd + '\n';
 
@@ -185,7 +251,9 @@ function REPLServer(prompt, stream, eval, useGlobal, ignoreUndefined) {
                 function(e, ret) {
             if (e && !isSyntaxError(e)) return finish(e);
 
-            if (typeof ret === 'function' || e) {
+            if (typeof ret === 'function' &&
+                /^[\r\n\s]*function/.test(evalCmd) ||
+                e) {
               // Now as statement without parens.
               self.eval(evalCmd, self.context, 'repl', finish);
             } else {
@@ -197,20 +265,21 @@ function REPLServer(prompt, stream, eval, useGlobal, ignoreUndefined) {
       finish(null);
     }
 
-    function isSyntaxError(e) {
-      // Convert error to string
-      e = e && (e.stack || e.toString());
-      return e && e.match(/^SyntaxError/) &&
-             !(e.match(/^SyntaxError: Unexpected token .*\n/) &&
-             e.match(/\n    at Object.parse \(native\)\n/));
-    }
-
     function finish(e, ret) {
 
       self.memory(cmd);
 
       // If error was SyntaxError and not JSON.parse error
       if (isSyntaxError(e)) {
+        if (!self.bufferedCommand && cmd.trim().match(/^npm /)) {
+          self.outputStream.write('npm should be run outside of the ' +
+                                  'node repl, in your normal shell.\n' +
+                                  '(Press Control-D to exit.)\n');
+          self.bufferedCmd = '';
+          self.displayPrompt();
+          return;
+        }
+
         // Start buffering data like that:
         // {
         // ...  x: 1
@@ -226,9 +295,9 @@ function REPLServer(prompt, stream, eval, useGlobal, ignoreUndefined) {
       self.bufferedCommand = '';
 
       // If we got any output - print it (if no error)
-      if (!e && (!ignoreUndefined || ret !== undefined)) {
+      if (!e && (!self.ignoreUndefined || ret !== undefined)) {
         self.context._ = ret;
-        self.outputStream.write(exports.writer(ret) + '\n');
+        self.outputStream.write(self.writer(ret) + '\n');
       }
 
       // Display prompt again
@@ -236,30 +305,32 @@ function REPLServer(prompt, stream, eval, useGlobal, ignoreUndefined) {
     };
   });
 
-  rli.addListener('close', function() {
-    self.inputStream.destroy();
+  rli.on('SIGCONT', function() {
+    self.displayPrompt(true);
   });
 
   self.displayPrompt();
 }
+inherits(REPLServer, EventEmitter);
 exports.REPLServer = REPLServer;
 
 
 // prompt is a string to print on each line for the prompt,
 // source is a stream to use for I/O, defaulting to stdin/stdout.
-exports.start = function(prompt, source, eval, useGlobal, ignoreUndefined) {
-  var repl = new REPLServer(prompt, source, eval, useGlobal, ignoreUndefined);
+exports.start = function(prompt, source, eval_, useGlobal, ignoreUndefined) {
+  var repl = new REPLServer(prompt, source, eval_, useGlobal, ignoreUndefined);
   if (!exports.repl) exports.repl = repl;
   return repl;
 };
 
 
 REPLServer.prototype.createContext = function() {
-  if (!this.useGlobal) {
-    var context = vm.createContext();
-    for (var i in global) context[i] = global[i];
+  var context;
+  if (this.useGlobal) {
+    context = global;
   } else {
-    var context = global;
+    context = vm.createContext();
+    for (var i in global) context[i] = global[i];
   }
 
   context.module = module;
@@ -273,42 +344,40 @@ REPLServer.prototype.createContext = function() {
   return context;
 };
 
-REPLServer.prototype.resetContext = function(force) {
-  if (!context || force) {
-    context = this.createContext();
-    for (var i in require.cache) delete require.cache[i];
+REPLServer.prototype.resetContext = function() {
+  for (var i in require.cache) delete require.cache[i];
+  this.context = this.createContext();
+};
+
+REPLServer.prototype.displayPrompt = function(preserveCursor) {
+  var prompt = this.prompt;
+  if (this.bufferedCommand.length) {
+    prompt = '...';
+    var levelInd = new Array(this.lines.level.length).join('..');
+    prompt += levelInd + ' ';
   }
-
-  this.context = context;
+  this.rli.setPrompt(prompt);
+  this.rli.prompt(preserveCursor);
 };
 
-REPLServer.prototype.displayPrompt = function() {
-  this.rli.setPrompt(this.bufferedCommand.length ?
-                 '...' + new Array(this.lines.level.length).join('..') + ' ' :
-                 this.prompt);
-  this.rli.prompt();
-};
-
-
-// read a line from the stream, then eval it
-REPLServer.prototype.readline = function(cmd) {
-};
 
 // A stream to push an array into a REPL
 // used in REPLServer.complete
 function ArrayStream() {
-  this.run = function (data) {
+  Stream.call(this);
+
+  this.run = function(data) {
     var self = this;
-    data.forEach(function (line) {
-      self.emit('data', line);
+    data.forEach(function(line) {
+      self.emit('data', line + '\n');
     });
   }
 }
-util.inherits(ArrayStream, require('stream').Stream);
+util.inherits(ArrayStream, Stream);
 ArrayStream.prototype.readable = true;
 ArrayStream.prototype.writable = true;
-ArrayStream.prototype.resume = function () {};
-ArrayStream.prototype.write = function () {};
+ArrayStream.prototype.resume = function() {};
+ArrayStream.prototype.write = function() {};
 
 var requireRE = /\brequire\s*\(['"](([\w\.\/-]+\/)?([\w\.\/-]*))/;
 var simpleExpressionRE =
@@ -332,7 +401,7 @@ REPLServer.prototype.complete = function(line, callback) {
     var tmp = this.lines.slice();
     // Kill off all function declarations to push all local variables into
     // global scope
-    this.lines.level.forEach(function (kill) {
+    this.lines.level.forEach(function(kill) {
       if (kill.isFunction) {
         tmp[kill.line] = '';
       }
@@ -422,13 +491,7 @@ REPLServer.prototype.complete = function(line, callback) {
     }
 
     if (!subdir) {
-      // Kind of lame that this needs to be updated manually.
-      // Intentionally excluding moved modules: posix, utils.
-      var builtinLibs = ['assert', 'buffer', 'child_process', 'crypto', 'dgram',
-        'dns', 'events', 'file', 'freelist', 'fs', 'http', 'net', 'os', 'path',
-        'querystring', 'readline', 'repl', 'string_decoder', 'util', 'tcp',
-        'url'];
-      completionGroups.push(builtinLibs);
+      completionGroups.push(exports._builtinLibs);
     }
 
     completionGroupsLoaded();
@@ -460,9 +523,6 @@ REPLServer.prototype.complete = function(line, callback) {
         expr = bits.join('.');
       }
 
-      // console.log("expression completion: completeOn='" + completeOn +
-      //             "' expr='" + expr + "'");
-
       // Resolve expr and get its completions.
       var obj, memberGroups = [];
       if (!expr) {
@@ -471,13 +531,17 @@ REPLServer.prototype.complete = function(line, callback) {
         if (this.useGlobal ||
             this.context.constructor &&
             this.context.constructor.name === 'Context') {
+          var contextProto = this.context;
+          while (contextProto = Object.getPrototypeOf(contextProto)) {
+            completionGroups.push(Object.getOwnPropertyNames(contextProto));
+          }
           completionGroups.push(Object.getOwnPropertyNames(this.context));
-          addStandardGlobals();
+          addStandardGlobals(completionGroups, filter);
           completionGroupsLoaded();
         } else {
           this.eval('.scope', this.context, 'repl', function(err, globals) {
             if (err || !globals) {
-              addStandardGlobals();
+              addStandardGlobals(completionGroups, filter);
             } else if (Array.isArray(globals[0])) {
               // Add grouped globals
               globals.forEach(function(group) {
@@ -485,34 +549,11 @@ REPLServer.prototype.complete = function(line, callback) {
               });
             } else {
               completionGroups.push(globals);
-              addStandardGlobals();
+              addStandardGlobals(completionGroups, filter);
             }
             completionGroupsLoaded();
           });
         }
-
-        function addStandardGlobals() {
-          // Global object properties
-          // (http://www.ecma-international.org/publications/standards/Ecma-262.htm)
-          completionGroups.push(['NaN', 'Infinity', 'undefined',
-            'eval', 'parseInt', 'parseFloat', 'isNaN', 'isFinite', 'decodeURI',
-            'decodeURIComponent', 'encodeURI', 'encodeURIComponent',
-            'Object', 'Function', 'Array', 'String', 'Boolean', 'Number',
-            'Date', 'RegExp', 'Error', 'EvalError', 'RangeError',
-            'ReferenceError', 'SyntaxError', 'TypeError', 'URIError',
-            'Math', 'JSON']);
-          // Common keywords. Exclude for completion on the empty string, b/c
-          // they just get in the way.
-          if (filter) {
-            completionGroups.push(['break', 'case', 'catch', 'const',
-              'continue', 'debugger', 'default', 'delete', 'do', 'else',
-              'export', 'false', 'finally', 'for', 'function', 'if',
-              'import', 'in', 'instanceof', 'let', 'new', 'null', 'return',
-              'switch', 'this', 'throw', 'true', 'try', 'typeof', 'undefined',
-              'var', 'void', 'while', 'with', 'yield']);
-          }
-        }
-
       } else {
         this.eval(expr, this.context, 'repl', function(e, obj) {
           // if (e) console.log(e);
@@ -525,7 +566,7 @@ REPLServer.prototype.complete = function(line, callback) {
             try {
               var sentinel = 5;
               var p;
-              if (typeof obj == 'object') {
+              if (typeof obj === 'object' || typeof obj === 'function') {
                 p = Object.getPrototypeOf(obj);
               } else {
                 p = obj.constructor ? obj.constructor.prototype : null;
@@ -561,6 +602,8 @@ REPLServer.prototype.complete = function(line, callback) {
     } else {
       completionGroupsLoaded();
     }
+  } else {
+    completionGroupsLoaded();
   }
 
   // Will be called when all completionGroups are in place
@@ -635,7 +678,7 @@ REPLServer.prototype.defineCommand = function(keyword, cmd) {
   this.commands['.' + keyword] = cmd;
 };
 
-REPLServer.prototype.memory = function memory (cmd) {
+REPLServer.prototype.memory = function memory(cmd) {
   var self = this;
 
   self.lines = self.lines || [];
@@ -654,7 +697,7 @@ REPLServer.prototype.memory = function memory (cmd) {
   // Because I can not tell the difference between a } that
   // closes an object literal and a } that closes a function
   if (cmd) {
-    // going down is { and (   e.g. function () {
+    // going down is { and (   e.g. function() {
     // going up is } and )
     var dw = cmd.match(/{|\(/g);
     var up = cmd.match(/}|\)/g);
@@ -663,18 +706,20 @@ REPLServer.prototype.memory = function memory (cmd) {
     var depth = dw - up;
 
     if (depth) {
-      (function workIt(){
+      (function workIt() {
         if (depth > 0) {
           // going... down.
           // push the line#, depth count, and if the line is a function.
           // Since JS only has functional scope I only need to remove
-          // "function () {" lines, clearly this will not work for
-          // "function ()
+          // "function() {" lines, clearly this will not work for
+          // "function()
           // {" but nothing should break, only tab completion for local
           // scope will not work for this function.
-          self.lines.level.push({ line: self.lines.length - 1,
-                                depth: depth,
-                                isFunction: /\s*function\s*/.test(cmd)});
+          self.lines.level.push({
+            line: self.lines.length - 1,
+            depth: depth,
+            isFunction: /\s*function\s*/.test(cmd)
+          });
         } else if (depth < 0) {
           // going... up.
           var curr = self.lines.level.pop();
@@ -706,6 +751,27 @@ REPLServer.prototype.memory = function memory (cmd) {
   }
 };
 
+function addStandardGlobals(completionGroups, filter) {
+  // Global object properties
+  // (http://www.ecma-international.org/publications/standards/Ecma-262.htm)
+  completionGroups.push(['NaN', 'Infinity', 'undefined',
+    'eval', 'parseInt', 'parseFloat', 'isNaN', 'isFinite', 'decodeURI',
+    'decodeURIComponent', 'encodeURI', 'encodeURIComponent',
+    'Object', 'Function', 'Array', 'String', 'Boolean', 'Number',
+    'Date', 'RegExp', 'Error', 'EvalError', 'RangeError',
+    'ReferenceError', 'SyntaxError', 'TypeError', 'URIError',
+    'Math', 'JSON']);
+  // Common keywords. Exclude for completion on the empty string, b/c
+  // they just get in the way.
+  if (filter) {
+    completionGroups.push(['break', 'case', 'catch', 'const',
+      'continue', 'debugger', 'default', 'delete', 'do', 'else',
+      'export', 'false', 'finally', 'for', 'function', 'if',
+      'import', 'in', 'instanceof', 'let', 'new', 'null', 'return',
+      'switch', 'this', 'throw', 'true', 'try', 'typeof', 'undefined',
+      'var', 'void', 'while', 'with', 'yield']);
+  }
+}
 
 function defineDefaultCommands(repl) {
   // TODO remove me after 0.3.x
@@ -729,7 +795,7 @@ function defineDefaultCommands(repl) {
       this.bufferedCommand = '';
       if (!this.useGlobal) {
         this.outputStream.write('Clearing context...\n');
-        this.resetContext(true);
+        this.resetContext();
       }
       this.displayPrompt();
     }
@@ -761,7 +827,7 @@ function defineDefaultCommands(repl) {
         fs.writeFileSync(file, this.lines.join('\n') + '\n');
         this.outputStream.write('Session saved to:' + file + '\n');
       } catch (e) {
-        this.outputStream.write('Failed to save:' + file+ '\n')
+        this.outputStream.write('Failed to save:' + file + '\n');
       }
       this.displayPrompt();
     }
@@ -777,7 +843,7 @@ function defineDefaultCommands(repl) {
           var data = fs.readFileSync(file, 'utf8');
           var lines = data.split('\n');
           this.displayPrompt();
-          lines.forEach(function (line) {
+          lines.forEach(function(line) {
             if (line) {
               self.rli.write(line + '\n');
             }
@@ -799,6 +865,7 @@ function trimWhitespace(cmd) {
   if (matches && matches.length === 2) {
     return matches[1];
   }
+  return '';
 }
 
 
@@ -834,3 +901,21 @@ REPLServer.prototype.convertToContext = function(cmd) {
 
   return cmd;
 };
+
+
+/**
+ * Returns `true` if "e" is a SyntaxError, `false` otherwise.
+ * This function filters out false positives likes JSON.parse() errors and
+ * RegExp syntax errors.
+ */
+function isSyntaxError(e) {
+  // Convert error to string
+  e = e && (e.stack || e.toString());
+  return e && e.match(/^SyntaxError/) &&
+      // RegExp syntax error
+      !e.match(/^SyntaxError: Invalid regular expression/) &&
+      !e.match(/^SyntaxError: Invalid flags supplied to RegExp constructor/) &&
+      // JSON.parse() error
+      !(e.match(/^SyntaxError: Unexpected (token .*|end of input)/) &&
+      e.match(/\n    at Object.parse \(native\)\n/));
+}
