@@ -22,16 +22,21 @@
 var crypto = require('crypto');
 var util = require('util');
 var net = require('net');
+var url = require('url');
 var events = require('events');
-var stream = require('stream');
+var Stream = require('stream');
 var END_OF_FILE = 42;
 var assert = require('assert').ok;
+var constants = require('constants');
+
+var DEFAULT_CIPHERS = 'ECDHE-RSA-AES128-SHA256:AES128-GCM-SHA256:' + // TLS 1.2
+                      'RC4:HIGH:!MD5:!aNULL:!EDH';                   // TLS 1.0
 
 // Allow {CLIENT_RENEG_LIMIT} client-initiated session renegotiations
 // every {CLIENT_RENEG_WINDOW} seconds. An error event is emitted if more
 // renegotations are seen. The settings are applied to all remote client
 // connections.
-exports.CLIENT_RENEG_LIMIT  = 3;
+exports.CLIENT_RENEG_LIMIT = 3;
 exports.CLIENT_RENEG_WINDOW = 600;
 
 
@@ -76,9 +81,140 @@ function convertNPNProtocols(NPNProtocols, out) {
   }
 }
 
+
+function checkServerIdentity(host, cert) {
+  // Create regexp to much hostnames
+  function regexpify(host, wildcards) {
+    // Add trailing dot (make hostnames uniform)
+    if (!/\.$/.test(host)) host += '.';
+
+    // The same applies to hostname with more than one wildcard,
+    // if hostname has wildcard when wildcards are not allowed,
+    // or if there are less than two dots after wildcard (i.e. *.com or *d.com)
+    if (/\*.*\*/.test(host) || !wildcards && /\*/.test(host) ||
+        /\*/.test(host) && !/\*.*\..+\..+/.test(host)) {
+      return /$./;
+    }
+
+    // Replace wildcard chars with regexp's wildcard and
+    // escape all characters that have special meaning in regexps
+    // (i.e. '.', '[', '{', '*', and others)
+    var re = host.replace(
+        /\*([a-z0-9\\-_\.])|[\.,\-\\\^\$+?*\[\]\(\):!\|{}]/g,
+        function(all, sub) {
+          if (sub) return '[a-z0-9\\-_]*' + (sub === '-' ? '\\-' : sub);
+          return '\\' + all;
+        });
+
+    return new RegExp('^' + re + '$', 'i');
+  }
+
+  var dnsNames = [],
+      uriNames = [],
+      ips = [],
+      valid = false;
+
+  // There're several names to perform check against:
+  // CN and altnames in certificate extension
+  // (DNS names, IP addresses, and URIs)
+  //
+  // Walk through altnames and generate lists of those names
+  if (cert.subjectaltname) {
+    cert.subjectaltname.split(/, /g).forEach(function(altname) {
+      if (/^DNS:/.test(altname)) {
+        dnsNames.push(altname.slice(4));
+      } else if (/^IP Address:/.test(altname)) {
+        ips.push(altname.slice(11));
+      } else if (/^URI:/.test(altname)) {
+        var uri = url.parse(altname.slice(4));
+        if (uri) uriNames.push(uri.hostname);
+      }
+    });
+  }
+
+  // If hostname is an IP address, it should be present in the list of IP
+  // addresses.
+  if (net.isIP(host)) {
+    valid = ips.some(function(ip) {
+      return ip === host;
+    });
+  } else {
+    // Transform hostname to canonical form
+    if (!/\.$/.test(host)) host += '.';
+
+    // Otherwise check all DNS/URI records from certificate
+    // (with allowed wildcards)
+    dnsNames = dnsNames.map(function(name) {
+      return regexpify(name, true);
+    });
+
+    // Wildcards ain't allowed in URI names
+    uriNames = uriNames.map(function(name) {
+      return regexpify(name, false);
+    });
+
+    dnsNames = dnsNames.concat(uriNames);
+
+    // And only after check if hostname matches CN
+    // (because CN is deprecated, but should be used for compatiblity anyway)
+    var commonNames = cert.subject.CN;
+    if (Array.isArray(commonNames)) {
+      for (var i = 0, k = commonNames.length; i < k; ++i) {
+        dnsNames.push(regexpify(commonNames[i], false));
+      }
+    } else {
+      dnsNames.push(regexpify(commonNames, false));
+    }
+
+    valid = dnsNames.some(function(re) {
+      return re.test(host);
+    });
+  }
+
+  return valid;
+}
+exports.checkServerIdentity = checkServerIdentity;
+
+
+
+function SlabBuffer() {
+  this.create();
+}
+
+
+SlabBuffer.prototype.create = function create() {
+  this.isFull = false;
+  this.pool = new Buffer(10 * 1024 * 1024);
+  this.offset = 0;
+  this.remaining = this.pool.length;
+};
+
+
+SlabBuffer.prototype.use = function use(context, fn) {
+  if (this.remaining === 0) {
+    this.isFull = true;
+    return 0;
+  }
+
+  var bytes = fn.call(context, this.pool, this.offset, this.remaining);
+
+  if (bytes > 0) {
+    this.offset += bytes;
+    this.remaining -= bytes;
+  }
+
+  assert(this.remaining >= 0);
+
+  return bytes;
+};
+
+
+var slabBuffer = new SlabBuffer();
+
+
 // Base class of both CleartextStream and EncryptedStream
 function CryptoStream(pair) {
-  stream.Stream.call(this);
+  Stream.call(this);
 
   this.pair = pair;
 
@@ -89,8 +225,9 @@ function CryptoStream(pair) {
   this._pending = [];
   this._pendingCallbacks = [];
   this._pendingBytes = 0;
+  this._buffer = slabBuffer;
 }
-util.inherits(CryptoStream, stream.Stream);
+util.inherits(CryptoStream, Stream);
 
 
 CryptoStream.prototype.write = function(data /* , encoding, cb */) {
@@ -165,13 +302,18 @@ CryptoStream.prototype.resume = function() {
 };
 
 
-CryptoStream.prototype.setTimeout = function(n) {
-  if (this.socket) this.socket.setTimeout(n);
+CryptoStream.prototype.setTimeout = function(timeout, callback) {
+  if (this.socket) this.socket.setTimeout(timeout, callback);
 };
 
 
-CryptoStream.prototype.setNoDelay = function() {
-  if (this.socket) this.socket.setNoDelay();
+CryptoStream.prototype.setNoDelay = function(noDelay) {
+  if (this.socket) this.socket.setNoDelay(noDelay);
+};
+
+
+CryptoStream.prototype.setKeepAlive = function(enable, initialDelay) {
+  if (this.socket) this.socket.setKeepAlive(enable, initialDelay);
 };
 
 
@@ -191,7 +333,14 @@ function parseCertString(s) {
     if (sepIndex > 0) {
       var key = parts[i].slice(0, sepIndex);
       var value = parts[i].slice(sepIndex + 1);
-      out[key] = value;
+      if (key in out) {
+        if (!Array.isArray(out[key])) {
+          out[key] = [out[key]];
+        }
+        out[key].push(value);
+      } else {
+        out[key] = value;
+      }
     }
   }
   return out;
@@ -326,12 +475,13 @@ CryptoStream.prototype._push = function() {
   }
 
   while (!this._paused) {
-    var bytesRead = 0;
-    var chunkBytes = 0;
-    var pool = new Buffer(16 * 4096); // alloc every time?
+    var chunkBytes = 0,
+        bytesRead = 0,
+        start = this._buffer.offset;
 
     do {
-      chunkBytes = this._pusher(pool, bytesRead, pool.length - bytesRead);
+      chunkBytes = this._buffer.use(this, this._pusher);
+      if (chunkBytes > 0) bytesRead += chunkBytes;
 
       if (this.pair.ssl && this.pair.ssl.error) {
         this.pair.error();
@@ -340,11 +490,12 @@ CryptoStream.prototype._push = function() {
 
       this.pair.maybeInitFinished();
 
-      if (chunkBytes >= 0) {
-        bytesRead += chunkBytes;
-      }
+    } while (chunkBytes > 0 && !this._buffer.isFull);
 
-    } while (chunkBytes > 0 && bytesRead < pool.length);
+    var pool = this._buffer.pool;
+
+    // Create new buffer if previous was filled up
+    if (this._buffer.isFull) this._buffer.create();
 
     assert(bytesRead >= 0);
 
@@ -356,7 +507,7 @@ CryptoStream.prototype._push = function() {
       return;
     }
 
-    var chunk = pool.slice(0, bytesRead);
+    var chunk = pool.slice(start, start + bytesRead);
 
     if (this === this.pair.cleartext) {
       debug('cleartext emit "data" with ' + bytesRead + ' bytes');
@@ -372,7 +523,7 @@ CryptoStream.prototype._push = function() {
     }
 
     // Optimization: emit the original buffer with end points
-    if (this.ondata) this.ondata(pool, 0, bytesRead);
+    if (this.ondata) this.ondata(pool, start, start + bytesRead);
   }
 };
 
@@ -469,9 +620,6 @@ CryptoStream.prototype._pull = function() {
       debug('drain ' + (this === this.pair.cleartext ? 'clear' : 'encrypted'));
       var self = this;
       process.nextTick(function() {
-        if (typeof self.ondrain === 'function') {
-          self.ondrain();
-        }
         self.emit('drain');
       });
       this._needDrain = false;
@@ -554,16 +702,14 @@ function onhandshakestart() {
   debug('onhandshakestart');
 
   var self = this, ssl = this.ssl;
-  ssl.handshakes++;
 
-  if (ssl.handshakes === 1) {
-    function timeout() {
+  if (ssl.timer === null) {
+    ssl.timer = setTimeout(function timeout() {
       ssl.handshakes = 0;
       ssl.timer = null;
-    }
-    ssl.timer = setTimeout(timeout, exports.CLIENT_RENEG_WINDOW * 1000);
+    }, exports.CLIENT_RENEG_WINDOW * 1000);
   }
-  else if (ssl.handshakes >= exports.CLIENT_RENEG_LIMIT) {
+  else if (++ssl.handshakes > exports.CLIENT_RENEG_LIMIT) {
     // Defer the error event to the next tick. We're being called from OpenSSL's
     // state machine and OpenSSL is not re-entrant. We cannot allow the user's
     // callback to destroy the connection right now, it would crash and burn.
@@ -906,18 +1052,20 @@ function Server(/* [options], listener */) {
 
   if (!(this instanceof Server)) return new Server(options, listener);
 
+  this._contexts = [];
+
   var self = this;
 
   // Handle option defaults:
   this.setOptions(options);
 
   var sharedCreds = crypto.createCredentials({
-    pfx : self.pfx,
+    pfx: self.pfx,
     key: self.key,
     passphrase: self.passphrase,
     cert: self.cert,
     ca: self.ca,
-    ciphers: self.ciphers || 'RC4-SHA:AES128-SHA:AES256-SHA',
+    ciphers: self.ciphers || DEFAULT_CIPHERS,
     secureProtocol: self.secureProtocol,
     secureOptions: self.secureOptions,
     crl: self.crl,
@@ -951,7 +1099,7 @@ function Server(/* [options], listener */) {
       } else {
         var verifyError = pair.ssl.verifyError();
         if (verifyError) {
-          pair.cleartext.authorizationError = verifyError;
+          pair.cleartext.authorizationError = verifyError.message;
 
           if (self.rejectUnauthorized) {
             socket.destroy();
@@ -1005,8 +1153,11 @@ Server.prototype.setOptions = function(options) {
   if (options.secureProtocol) this.secureProtocol = options.secureProtocol;
   if (options.crl) this.crl = options.crl;
   if (options.ciphers) this.ciphers = options.ciphers;
-  if (options.secureProtocol) this.secureProtocol = options.secureProtocol;
-  if (options.secureOptions) this.secureOptions = options.secureOptions;
+  var secureOptions = options.secureOptions || 0;
+  if (options.honorCipherOrder) {
+    secureOptions |= constants.SSL_OP_CIPHER_SERVER_PREFERENCE;
+  }
+  if (secureOptions) this.secureOptions = secureOptions;
   if (options.NPNProtocols) convertNPNProtocols(options.NPNProtocols, this);
   if (options.SNICallback) {
     this.SNICallback = options.SNICallback;
@@ -1023,7 +1174,6 @@ Server.prototype.setOptions = function(options) {
 };
 
 // SNI Contexts High-Level API
-Server.prototype._contexts = [];
 Server.prototype.addContext = function(servername, credentials) {
   if (!servername) {
     throw 'Servername is required parameter for Server.addContext';
@@ -1052,7 +1202,7 @@ Server.prototype.SNICallback = function(servername) {
 
 // Target API:
 //
-//  var s = tls.connect(8000, "google.com", options, function() {
+//  var s = tls.connect({port: 8000, host: "google.com"}, function() {
 //    if (!s.authorized) {
 //      s.destroy();
 //      return;
@@ -1064,35 +1214,36 @@ Server.prototype.SNICallback = function(servername) {
 //  });
 //
 //
-// TODO:  make port, host part of options!
-exports.connect = function(port /* host, options, cb */) {
-  // parse args
-  var host, options = {}, cb;
-  for (var i = 1; i < arguments.length; i++) {
-    switch (typeof arguments[i]) {
-      case 'string':
-        host = arguments[i];
-        break;
+function normalizeConnectArgs(listArgs) {
+  var args = net._normalizeConnectArgs(listArgs);
+  var options = args[0];
+  var cb = args[1];
 
-      case 'object':
-        options = arguments[i];
-        break;
-
-      case 'function':
-        cb = arguments[i];
-        break;
-    }
+  if (typeof listArgs[1] === 'object') {
+    options = util._extend(options, listArgs[1]);
+  } else if (typeof listArgs[2] === 'object') {
+    options = util._extend(options, listArgs[2]);
   }
 
+  return (cb) ? [options, cb] : [options];
+}
+
+exports.connect = function(/* [port, host], options, cb */) {
+  var args = normalizeConnectArgs(arguments);
+  var options = args[0];
+  var cb = args[1];
+
   var socket = options.socket ? options.socket : new net.Stream();
 
   var sslcontext = crypto.createCredentials(options);
 
   convertNPNProtocols(options.NPNProtocols, this);
-  var pair = new SecurePair(sslcontext, false, true, false,
+  var hostname = options.servername || options.host || 'localhost',
+      pair = new SecurePair(sslcontext, false, true,
+                            options.rejectUnauthorized === true ? true : false,
                             {
                               NPNProtocols: this.NPNProtocols,
-                              servername: options.servername || host
+                              servername: hostname
                             });
 
   if (options.session) {
@@ -1101,11 +1252,16 @@ exports.connect = function(port /* host, options, cb */) {
 
   var cleartext = pipe(pair, socket);
   if (cb) {
-    cleartext.on('secureConnect', cb);
+    cleartext.once('secureConnect', cb);
   }
 
   if (!options.socket) {
-    socket.connect(port, host);
+    var connect_opt = (options.path && !options.port) ? {path: options.path} : {
+      port: options.port,
+      host: options.host,
+      localAddress: options.localAddress
+    };
+    socket.connect(connect_opt);
   }
 
   pair.on('secure', function() {
@@ -1113,14 +1269,30 @@ exports.connect = function(port /* host, options, cb */) {
 
     cleartext.npnProtocol = pair.npnProtocol;
 
-    if (verifyError) {
-      cleartext.authorized = false;
-      cleartext.authorizationError = verifyError;
-    } else {
-      cleartext.authorized = true;
+    // Verify that server's identity matches it's certificate's names
+    if (!verifyError) {
+      var validCert = checkServerIdentity(hostname,
+                                          pair.cleartext.getPeerCertificate());
+      if (!validCert) {
+        verifyError = new Error('Hostname/IP doesn\'t match certificate\'s ' +
+                                'altnames');
+      }
     }
 
-    cleartext.emit('secureConnect');
+    if (verifyError) {
+      cleartext.authorized = false;
+      cleartext.authorizationError = verifyError.message;
+
+      if (pair._rejectUnauthorized) {
+        cleartext.emit('error', verifyError);
+        pair.destroy();
+      } else {
+        cleartext.emit('secureConnect');
+      }
+    } else {
+      cleartext.authorized = true;
+      cleartext.emit('secureConnect');
+    }
   });
   pair.on('error', function(err) {
     cleartext.emit('error', err);
